"""Pagination module."""

from dataclasses import dataclass
from typing import Generic, TypedDict, TypeVar

from wriftai.common_types import NotRequired

T = TypeVar("T")


class PaginationOptions(TypedDict):
    """Options for pagination.

    Attributes:
        cursor (NotRequired[str]): Cursor for pagination.
        page_size (NotRequired[int]): Number of items per page.
    """

    cursor: NotRequired[str]
    page_size: NotRequired[int]


@dataclass
class PaginatedResponse(Generic[T]):
    """Represents a paginated response.

    Attributes:
        items (list[T]): List of items returned in the current page.
        next_cursor (str | None): Cursor pointing to the next page.
        previous_cursor (str | None): Cursor pointing to the previous page.
        next_url (str | None): URL to fetch the next page.
        previous_url (str | None): URL to fetch the previous page.
    """

    items: list[T]
    next_cursor: str | None
    previous_cursor: str | None
    next_url: str | None
    previous_url: str | None



"""Common types used across the WriftAI package."""

import sys
from collections.abc import Mapping
from typing import Any, TypeAlias, TypedDict, Union

__all__ = ["NotRequired", "StrEnum", "Unpack"]


JsonValue: TypeAlias = Union[
    list["JsonValue"],
    Mapping[str, "JsonValue"],
    str,
    bool,
    int,
    float,
    None,
]

if sys.version_info >= (3, 11):
    from enum import StrEnum
    from typing import NotRequired, Unpack
else:
    from enum import Enum

    from typing_extensions import NotRequired, Unpack

    class StrEnum(str, Enum):
        """String-based enum class for compatibility with older Python versions."""

        pass


class User(TypedDict):
    """Represents a user.

    Attributes:
            id (str): Unique identifier of the user.
            username (str): The username of the user.
            avatar_url (str): URL of the user's avatar.
            name (str | None): The name of the user.
            bio (str | None): The biography of the user.
            urls (list[str] | None): Personal or professional website URLs.
            location (str | None): Location of the user.
            company (str | None): Company the user is associated with.
            created_at (str): Timestamp when the user joined WriftAI.
            updated_at (str | None): Timestamp when the user was last updated.
    """

    id: str
    username: str
    avatar_url: str
    name: str | None
    bio: str | None
    urls: list[str] | None
    location: str | None
    company: str | None
    created_at: str
    updated_at: str | None


class SchemaIO(TypedDict):
    """Represents input and output schemas.

    Attributes:
        input (dict[str, Any]): Schema for input, following JSON Schema
            Draft 2020-12 standards.
        output (dict[str, Any]): Schema for output, following JSON Schema
            Draft 2020-12 standards.
    """

    input: dict[str, Any]
    output: dict[str, Any]


class Schemas(TypedDict):
    """Represents schemas of a version.

    Attributes:
        prediction (SchemaIO):The input and output schemas for a prediction.
    """

    prediction: SchemaIO


class Version(TypedDict):
    """Represents a version.

    Attributes:
        id (str):The unique identifier of the version.
        release_notes (str): Information about changes such as
            new features,bug fixes, or optimizations in this version.
        created_at (str): The time when the version was created.
        schemas (Schemas): The schemas of the model version.
        container_image_digest (str): A sha256 hash digest of
            the version's container image.
    """

    id: str
    release_notes: str
    created_at: str
    schemas: Schemas
    container_image_digest: str



from typing import Callable

import httpx
import pytest
from respx import Router

from tests.integration.conftest import Route
from wriftai.client import Client, ClientOptions
from wriftai.pagination import PaginatedResponse


@pytest.mark.asyncio
@pytest.mark.parametrize("async_flag", [True, False])
async def test_list(mock_router: Callable[..., Router], async_flag: bool) -> None:
    expected_json = {
        "items": [
            {
                "id": "551c7d27-11e8-40a4-90d3-e8d8c22c5894",
                "name": "NVIDIA A100",
                "gpus": 8,
                "cpus": 64,
                "ram_per_gpu_gb": 40,
                "ram_gb": 320,
                "created_at": "2025-08-13T11:48:44.371093Z",
            }
        ],
        "next_cursor": "abc123",
        "previous_cursor": None,
        "next_url": "/hardware?cursor=abc123",
        "previous_url": None,
    }

    router = mock_router(
        route=Route(
            method="GET",
            path="/hardware",
            status_code=200,
            json=expected_json,
        )
    )

    client = Client(
        client_options=ClientOptions(
            headers={"Authorization": "test-token"},
            timeout=httpx.Timeout(15),
            transport=httpx.MockTransport(router.handler),
        )
    )

    if async_flag:
        response = await client.hardware.async_list()
    else:
        response = client.hardware.list()

    # expected_json is a dictionary.
    # Although the structure matches the expected fields, static type checkers like mypy
    # may not be able to verify this due to dynamic typing.
    assert response == PaginatedResponse(**expected_json)  # type:ignore[arg-type]
